local B, L, T = unpack(select(2, ...))

local fontName, fontHeight
local function getfont()

    fontName, fontHeight = T.F.moneyframe.label:GetFont()
    return fontHeight
end


B.M = {}
function B.M.Create()

    local moneyframe = CreateFrame("Frame", "BWmoneyframe", UIParent)
    moneyframe:SetPoint("CENTER", 0, 200)
    moneyframe:SetHeight(50)
    moneyframe:SetWidth(200)
    moneyframe:SetMovable(true)
    moneyframe:SetUserPlaced(true)
    moneyframe:SetClampedToScreen(true)
    moneyframe:Hide()
    moneyframe:SetFrameStrata("DIALOG")

    local moneyframe_anchor = CreateFrame("Frame", "BWmoneyanchor", UIParent)
    moneyframe_anchor:SetAllPoints(moneyframe)
    moneyframe_anchor:SetHeight(moneyframe:GetHeight())
    moneyframe_anchor:SetWidth(moneyframe:GetWidth())
    moneyframe_anchor:SetMovable(true)
    moneyframe_anchor:SetUserPlaced(true)
    moneyframe_anchor:SetClampedToScreen(true)
    B.I.Backdrop(moneyframe_anchor)
    moneyframe_anchor:Hide()
    moneyframe_anchor:SetFrameStrata("DIALOG")

    moneyframe_anchor:SetScript("OnMouseDown", B.M.Frame.MouseDown)
    moneyframe_anchor:SetScript("OnMouseUp", B.M.Frame.MouseUp)
    moneyframe.anchor = moneyframe_anchor

    local label_anchor = moneyframe_anchor:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    label_anchor:SetAllPoints()
    label_anchor:SetJustifyV("BOTTOM")
    label_anchor:SetText(L[10052])

    local label = moneyframe:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    label:SetTextColor(1, 1, 1)
    label:SetFont(T.FONTS[1], 18)
    label:SetPoint("CENTER")
    label:SetJustifyH("CENTER")
    label:SetJustifyV("TOP")
    label:SetWordWrap(false)
    moneyframe.label = label

    local totalmoney = moneyframe:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    totalmoney:SetPoint("BOTTOM", label, "TOP", 0, 1)
    totalmoney:SetFont(T.FONTS[2], 14)
    totalmoney:SetJustifyH("CENTER")
    totalmoney:SetJustifyV("CENTER")
    totalmoney:SetWordWrap(false)
    moneyframe.totalmoney = totalmoney

    moneyframe:SetScript("OnShow", B.M.Frame.Show)
    moneyframe:SetScript("OnHide", B.M.Frame.Hide)

    T.F.moneyframe = moneyframe
    -- load font label
    getfont()
end

B.M.Frame = {}
local pulse = 2
local pulsing = 1 / pulse
local point, relativeTo, relativePoint, xOffset, yOffset
local current, size

function B.M.Frame:Update(elapsed)
    if not elapsed then return end

    point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint()
    if not self.offset then self.offset = {point, relativeTo, relativePoint, xOffset, yOffset} end

    self.elapsed = self.elapsed + elapsed 
    if math.abs(self.offset[5] - yOffset) < 45 then self:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset + 4) end

    current, size = T.F.moneyframe.label:GetFont()
    if not self.defaultfontsize then self.defaultfontsize = size end

    if not self.hide then
        T.F.moneyframe.label:SetFont(current, math.min(size + pulsing, self.defaultfontsize + 5))
    else
        T.F.moneyframe.label:SetFont(current, math.max(size - pulsing, self.defaultfontsize))
        self:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset - 0.5)
    end

    if self.hide and self.elapsed > 0.7 then
        self:Hide()

    elseif not self.hide and self.elapsed > 2 then
        UIFrameFadeOut(self, 0.7, 1, 0)
        T.BW.total_elapsed = 0
        self.elapsed = 0
        self.hide = true
    end
end

function B.M.Frame:Show()
    self.elapsed = 0
    self.hide = nil
    self:SetScript("OnUpdate", B.M.Frame.Update)
end

function B.M.Frame:Hide()
    self:SetScript("OnUpdate", nil)
    self:SetPoint(unpack(self.offset))
    self.label:SetFont(self.label:GetFont(), 18)
    self.defaultfontsize = nil
    self.elapsed = nil
    self.hide = nil
end

function B.M.Frame:MouseDown(button)
    if button == "LeftButton" then
        self:StartMoving()
    elseif button == "RightButton" then
        self:Hide()
        T.F.moneyframe:Hide()
    end
end

function B.M.Frame:MouseUp(button)
    self:StopMovingOrSizing()
    T.F.moneyframe:ClearAllPoints()
    point, relativeTo, relativePoint, xOffset, yOffset = self:GetPoint()
    T.F.moneyframe.offset = {point, relativeTo, relativePoint, xOffset, yOffset}
    T.F.moneyframe:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset)
end

function B.M.Frame.Scale(scale)

    T.F.moneyframe:SetScale(scale / 100)
    T.F.moneyframe.anchor:SetScale(scale / 100)
end


local money_old = 0
local money_cumulative = 0
local difference = 0
local treasure_count = 0
local money_count = 0

function B.M.Trigger(test)

    if T.CONFIG['enablemoneytracker'] or test then
        if money_cumulative == 0 then T.F.moneyframe:Hide() end

        if money_old == 0 then money_old = T['currency']['money'] end
        T['currency']['money'] = GetMoney()
        T.F.moneyframe.totalmoney:SetText(GetMoneyString(B.round(T['currency']['money']), true))

        difference = money_old - T['currency']['money'] + tonumber(test or 0)
        money_old = T['currency']['money']

        if difference < 0 or test then -- Gained money!
            T.BW.total_elapsed = 0
            money_count = money_count + 1

            if not BWframe:GetScript("OnUpdate") then BWframe:SetScript("OnUpdate", B.M.Counter.Update) end

            money_cumulative = money_cumulative + math.abs(difference)
            T.F.moneyframe.label:SetText("+"..GetMoneyString(money_cumulative, true))

            if money_cumulative > difference then -- Force to remain displayed
                getfont()

                if money_cumulative >= T.CONFIG['treasuresoundtracker'] then -- blow it up anyway
                    fontHeight = 26
                    T.F.moneyframe.label:SetFont(fontName, fontHeight)
                else
                    fontHeight = math.min(fontHeight + 0.2, 26)
                    T.F.moneyframe.label:SetFont(fontName, fontHeight)
                end
                
                T.F.moneyframe.label:SetText("+"..GetMoneyString(money_cumulative, true))
                T.F.moneyframe.totalmoney:SetText(GetMoneyString(B.round(T['currency']['money']), true))
                T.F.moneyframe.elapsed = 0
                T.F.moneyframe.hide = nil
                T.F.moneyframe.defaultfontsize = fontHeight

                if money_cumulative >= T.CONFIG['treasuresoundtracker'] then
                    if treasure_count == 5 then -- force reset sound flag
                        T.F.moneyframe.sound = nil
                        treasure_count = 0
                    else
                        treasure_count = treasure_count + 1
                    end
                end
            end

            if T.CONFIG['enablesoundsmoneytracker'] then
                if not T.CONFIG['minimumsoundtracker'] or T.CONFIG['minimumsoundtracker'] < money_cumulative then
                    PlaySoundFile(T.SOUNDS[T.CONFIG['moneysound']], "Master")
                end
            end
            UIFrameFadeIn(T.F.moneyframe, 0.3, 0.3, 1)
        end
    end
end

function B.M.TreasureTrigger()

    if money_cumulative >= T.CONFIG['treasuresoundtracker'] then
        T.F.moneyframe.sound = true
        if T.CONFIG['enablesoundsmoneytracker'] then PlaySoundFile(T.SOUNDS[T.CONFIG['treasuresound']], "Master") end
    end
end

function B.M.Reset()

    if money_cumulative > 0 then
        money_cumulative = 0
        treasure_count = 0
        money_count = 0
        T.F.moneyframe.sound = nil
        T.F.moneyframe.label:SetFont(T.FONTS[1], 18)
    end
end

B.M.Counter = {}
function B.M.Counter:Update(elapsed)

    if not elapsed then return end -- Otherwise it will infinite loop

    T.BW.total_elapsed = T.BW.total_elapsed + elapsed

    if not T.F.moneyframe.sound and T.BW.total_elapsed > 1 then
        B.M.TreasureTrigger()
    end

    if T.BW.total_elapsed > 2.5 then
        B.M.Reset()
        BWframe:SetScript("OnUpdate", nil)
    end
end
